=begin
  version - version information of gettext

  Copyright (C) 2012-2023  Sutou Kouhei <kou@clear-code.com>
  Copyright (C) 2005-2009  Masao Mutoh

  You may redistribute it and/or modify it under the same
  license terms as Ruby or LGPL.
=end

module GetText
  VERSION = "3.4.9"
end
